/*******************************************************************************
*
*	TITLE:		breaker_panel.c 
*
*	VERSION:	1.1                           
*
*	DATE:		25-Jan-2005
*
*	AUTHOR:		IFI Robotics
*
*				R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:
*
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	             1.0  IFI Robotics' original code
*	25-Jan-2005  1.1  RKW - moved breaker panel code from user_SerialDrv.c/.h
*	                  to breaker_panel.c/.h.
*
*******************************************************************************/
#include "ifi_default.h"
#include "breaker_panel.h"

unsigned char IntState;
PANEL_INPUT_RECORD  dataIn;
unsigned char aBreakerWasTripped = FALSE;

/*******************************************************************************
* FUNCTION NAME: Breaker_Tripped
* PURPOSE:       This routine determines the tripped breaker(s) using the data
*                from the breaker panel. 
*
* NOTE:          Normally device specific code should reside in another file but
*                for the purpose of simplification we chose to put it here. 
*
* CALLED FROM:   CheckUartInts
* ARGUMENTS:     1
*     Argument       Type             IO   Description
*     --------       -------------    --   -----------
*     id             unsigned char    I    breaker id number
*******************************************************************************/
unsigned char Breaker_Tripped(unsigned char id)
{
  unsigned char bit2check;
  unsigned char retn;

  retn = FALSE;
  if (id < 9)         //Check breakers 1-8
  {
    bit2check = 1 << (id - 1);
    retn = dataIn.data4 & bit2check;
  }
  else if (id < 17)   //Check breakers 9-16
  {
    bit2check = 1 << (id - 9);
    retn = dataIn.data3 & bit2check;
  }
  else if (id < 25)   //Check breakers 17-24
  {
    bit2check = 1 << (id - 17);
    retn = dataIn.data2 & bit2check;
  }
  else                //Check breakers 25-28
  {
    bit2check = 1 << (id - 25);
    retn = dataIn.data1Byte.allbits & bit2check;
  }
  return retn;
}

/*******************************************************************************
* FUNCTION NAME: Handle_Panel_Data
* PURPOSE:       State machine for incomming Breaker Panel data.  
*
*
* NOTE:          Normally device specific code should reside in another file but
*                for the purpose of simplification we chose to put it here. 
*
* CALLED FROM:   CheckUartInts
* ARGUMENTS:     1
*     Argument       Type             IO   Description
*     --------       -------------    --   -----------
*     data           unsigned char    I    incoming data byte
* LIMITATIONS:   The data from the breaker panel does not contain a checksum.
*				 See the breaker-panel-packet-definitions.pdf file for more info.
*******************************************************************************/

void Handle_Panel_Data(unsigned char data)  
{
  switch (IntState)
  {
    case 0:  // 1st 0xC9
      IntState = 0;
      if (data == 0xC9) IntState = 1;
      break;
    case 1 : // 2nd 0x17
      IntState = 0;
      if (data == 0x17) IntState = 2;
      break;
    case 2 : //get DATA1
      dataIn.data1Byte.allbits = data;
      txdata.user_byte3 = data;
      aBreakerWasTripped = (int) dataIn.data1Byte.bitselect.tripped;
      IntState = 3;
      break;
    case 3 : //get DATA2
      dataIn.data2 = data;
      txdata.user_byte4 = data;
      IntState = 4;
      break;
    case 4 : //get DATA3
      dataIn.data3 = data;
      txdata.user_byte5 = data;
      IntState = 5;
      break;
    case 5 : //get DATA4
      dataIn.data4 = data;
      txdata.user_byte6 = data;
      IntState = 0;
      break;
    default:
      IntState= 0;
      break;
  }
}
